/*
*	Filename:		AnimationExport.cpp
*	Author:			Lars Vidar Magnusson
*	Copyright (C)	LaViMa Development 2006
*/

#include "AnimationExport.h"

bool operator <(tKeyFrame left, tKeyFrame right) {
	return left.time < right.time;
}

std::set<MTime> cAnimationExport::findKeyFrames(MDagPath &jointPath, MStatus *status) {
	std::set<MTime> keyFrames;
	
	// Find the animations for this joint
	MPlugArray animatedPlugs;
  MAnimUtil::findAnimatedPlugs(jointPath, animatedPlugs, false, status);
	
  for(unsigned int i = 0; i<animatedPlugs.length(); ++i) {
	    
		MObjectArray curves;
		MAnimUtil::findAnimation(animatedPlugs[i], curves, status);
		
		for (unsigned int j = 0; j < curves.length(); ++j) {
      
			MFnAnimCurve curve(curves[j], status);
			
			for(unsigned int k=0; k<curve.numKeys(); ++k) {
				keyFrames.insert(curve.time(k));
	    }
		}
	}
	return keyFrames;
}

MStatus cAnimationExport::extractAnimation() {
	MStatus status = MStatus::kSuccess;

	mAnimation.numFrames = 0;
	
	unsigned int jointIndex = 0;
	MItDag dagIter(MItDag::kDepthFirst, MFn::kJoint, &status);
  for( ; !dagIter.isDone(); dagIter.next(), jointIndex++) {

		MDagPath jointPath;
		status = dagIter.getPath(jointPath);

		if(MAnimUtil::isAnimated(jointPath)) {
			tAnimationTrack newTrack;
			newTrack.jointName = jointPath.partialPathName(&status);
			newTrack.jointIndex = jointIndex;
			
			std::set<MTime> keyFrames = findKeyFrames(jointPath, &status);

			std::set<MTime>::iterator iter;
			for(iter=keyFrames.begin(); iter!= keyFrames.end(); ++iter) {
				tKeyFrame newKeyFrame;
				newKeyFrame.time = *iter;

				if(newKeyFrame.time.value() > mAnimation.numFrames) {
					mAnimation.numFrames = newKeyFrame.time.value();
				}
				
				MAnimControl::setCurrentTime(*iter);
				
				MFnTransform fnTransform(jointPath, &status);
				MFnIkJoint fnJoint(jointPath, &status);
								
				status = fnTransform.getRotation(newKeyFrame.rotation, MSpace::kTransform);
				
				newTrack.keyFrames.push_back(newKeyFrame);
			}
			mAnimation.tracks.push_back(newTrack);
		}		
	}	
	return status;
}

void cAnimationExport::writeAnimation(cTextFileWriter &writer) {
	writer.addTab();
	writer.add("animation %.0f %i\n", mAnimation.numFrames, mAnimation.tracks.size());
	
	for(unsigned int track=0; track<mAnimation.tracks.size(); ++track) {
		writer.addTab();
		writer.add("track %i %i \"%s\"\n", track, mAnimation.tracks[track].jointIndex, mAnimation.tracks[track].jointName.asChar());
		writer.addTab(); 
		writer.add("keyframes %i\n", mAnimation.tracks[track].keyFrames.size());
		for(unsigned int frame=0; frame<mAnimation.tracks[track].keyFrames.size(); ++frame) {
			writer.add("%i %.0f %f %f %f %f\n", frame, mAnimation.tracks[track].keyFrames[frame].time.value(), 
				mAnimation.tracks[track].keyFrames[frame].rotation.x, mAnimation.tracks[track].keyFrames[frame].rotation.y, 
				mAnimation.tracks[track].keyFrames[frame].rotation.z, mAnimation.tracks[track].keyFrames[frame].rotation.w);
		}
		writer.removeTab();
		writer.removeTab();
		writer.add("\n");
	}

	writer.removeTab();
	writer.add("\n");
}

void cAnimationExport::cleanUp() {
	mAnimation.tracks.clear();
}