/*
*	Filename:		MeshExport.h
*	Author:			Lars Vidar Magnusson
*	Copyright (C)	LaViMa Development 2006
*/

#ifndef __MESHEXPORT__
#define __MESHEXPORT__

#include <maya/MStatus.h>
#include <maya/MDagPath.h>
#include <maya/MDagPathArray.h>
#include <maya/MItDag.h>
#include <maya/MItDependencyNodes.h>
#include <maya/MItGeometry.h>
#include <maya/MItMeshPolygon.h>
#include <maya/MPointArray.h>
#include <maya/MFloatArray.h>
#include <maya/MFloatVectorArray.h>
#include <maya/MFnMesh.h>
#include <maya/MFnDagNode.h>
#include <maya/MFnSkinCluster.h>
#include <maya/MFnLambertShader.h>
#include <maya/MItDependencyGraph.h>

#include <TextFileWriter.h>
#include <Mesh.h>

#include <vector>

class cMeshExport {
private:

	std::vector<tMesh *> mMeshes;
	std::vector<MString> mAlreadyExported;

	std::vector<tShader *> mShaders;


	// Helper function

	bool isAlreadyExported(MDagPath &meshPath);

	MStatus extractVertices(MFnMesh &fnMesh, tMesh *mesh);

	MStatus extractNormals(MFnMesh &fnMesh, tMesh *mesh);

	MStatus extractUVs(MFnMesh &fnMesh, tMesh *mesh);

	MStatus extractWeights(MDagPath meshPath, tMesh *mesh);

	MStatus extractTriangles(MFnMesh &fnMesh, tMesh *mesh, MIntArray &attributes);

	MStatus extractMaterials(MObjectArray &shaders);

	MStatus extractMesh(MDagPath &meshPath);

public:

  /**
	*	Constructor
	*/
	cMeshExport();

	/**
	*	Destructor
	*/
	~cMeshExport();

	MStatus extractMeshes();	

	void writeMeshes(cTextFileWriter &writer);

	void writeMaterials(cTextFileWriter &writer);

	void cleanUp();

};	// MeshExport

#endif	// __MESHEXPORT__