/*
*	Filename:		RiTETranslator.h
*	Author:			Lars Vidar Magnusson
*	Copyright (C)	LaViMa Development 2005
*/

#ifndef __RITETRANSLATOR__
#define __RITETRANSLATOR__

#include <maya/MPxFileTranslator.h>
#include <TextFileWriter.h>
#include <MeshExport.h>
#include "SkeletonExport.h"
#include "AnimationExport.h"

class cRiTETranslator : public MPxFileTranslator {

	cMeshExport mMeshExport;
	cSkeletonExport mSkeletonExport;
	cAnimationExport mAnimationExport;
	
public:

	// The version of the translator
	static MString mVersion;

	/**
	*	Creator method. Creates the instance of the translator
	*/
	static void *creator();

	/**
	*	Reader method
	*	@remark 
	*		This function is not implemented.
	* @par
	*		This method is inherited from MPxFileTranslator
	*/
	MStatus reader(const MFileObject &file, const MString& optionsString, MPxFileTranslator::FileAccessMode mode);

	/**
	*	Writer method.
	* @par
	*		This method is inherited from MPxFileTranslator.
	*/
	MStatus writer(const MFileObject &fileObject, const MString &options, MPxFileTranslator::FileAccessMode mode);

	/**
	*	Test if the translator has a reader function.
	* @par
	*		This method is inherited from MPxFileTranslator
	*/
	bool haveReadMethod () const;

	/**
	*	Test if the translator has a writer function.
	* @par
	*		This method is inherited from MPxFileTranslator
	*/
	bool haveWriteMethod () const;
	
	/**
	*	Return the default extension of the translator
	* @par
	*		This method is inherited from MPxFileTranslator
	*/
	MString defaultExtension () const;
	
	/**
	*	Return wether the translator can be used to open a file.
	* @par
	*		This method is inherited from MPxFileTranslator
	*	@return
	*		Returns true if the translator can be used to both open and 
	*		import files. False if only import is supported
	*/
	bool canBeOpened () const;

	/**
	*	Method for identifying a supported file
	* @par
	*		This method is inherited from MPxFileTranslator
	*	@return 
	*		An enumeration that tells if the file is supported or not.
	*/
	MFileKind identifyFile(const MFileObject &file,	const char *buffer, short size) const;

	/**
	*	Not sure what this method is for
	* @par
	*		This method is inherited from MPxFileTranslator
	*	@return 
	*		False
	*/
	bool haveNamespaceSupport() const;

	/**
	*	Not sure what this method is for
	* @par
	*		This method is inherited from MPxFileTranslator
	*	@return 
	*		An empty string
	*/
	MString filter() const;

};	// RiTETranslator

#endif	// __RITETRANSLATOR__